import logging
from typing import Optional, Tuple, TYPE_CHECKING

import claripy
from archinfo import Arch
from angr.sim_type import SimType, SimTypeFunction
from angr.analyses.reaching_definitions.subject import Subject
from angr.analyses.reaching_definitions.call_trace import CallTrace
from angr.calling_conventions import SimRegArg, SimStackArg, SimCC, SimFunctionArgument
from angr.engines.light import SpOffset
from angr.knowledge_plugins import Function
from angr.knowledge_plugins.key_definitions.atoms import Register, MemoryLocation
from angr.knowledge_plugins.key_definitions.definition import Definition
from angr.knowledge_plugins.key_definitions.tag import ParameterTag, InitialValueTag
from angr.code_location import ExternalCodeLocation

l = logging.getLogger(name=__name__)

if TYPE_CHECKING:
    from angr.analyses.reaching_definitions.rd_state import ReachingDefinitionsState


class RDAStateInitializer:
    """
    This class acts as the basic implementation for the logic that initializes the base state
    for the reaching definitions analysis.

    It also defines the _interface_ that a state initializer should implement,
    if the language/runtime being analyzed requires more complicated logic to set up the state.

    This code/logic was previously part of the ReachingDefinitionsState class, but this was moved here to separate
    these two concerns, and allow easily changing the initialization logic without having to change the state class.

    """

    def __init__(self, arch: Arch):
        self.arch: Arch = arch

    def initialize_function_state(
        self, state: "ReachingDefinitionsState", cc: Optional[SimCC], func_addr: int, rtoc_value: Optional[int] = None
    ) -> None:
        """
        This is the entry point to the state initialization logic.
        It will be called during the initialization of an ReachingDefinitionsState,
        if the state was freshly created (without existing live_definitions)
        """
        call_string = self._generate_call_string(state._subject, func_addr)
        ex_loc = ExternalCodeLocation(call_string)
        if state.analysis is not None:
            state.analysis.model.at_new_stmt(ex_loc)

        # Setup Stack pointer
        self.initialize_stack_pointer(state, func_addr, ex_loc)

        # initialize function arguments, based on the calling convention and signature
        if state.analysis is not None and cc is not None:
            prototype = state.analysis.kb.functions[func_addr].prototype
        else:
            prototype = None
        self.initialize_all_function_arguments(state, func_addr, ex_loc, cc, prototype)

        # architecture dependent initialization
        self.initialize_architectural_state(state, func_addr, ex_loc, rtoc_value)

        if state.analysis is not None:
            state.analysis.model.complete_loc()

    def initialize_all_function_arguments(
        self,
        state: "ReachingDefinitionsState",
        func_addr: int,
        ex_loc: ExternalCodeLocation,
        cc: Optional[SimCC],
        prototype: Optional[SimTypeFunction],
    ) -> None:
        """
        This method handles the setup for _all_ arguments of a function.

        The default implementation uses the calling convention to extract the argument locations, associates them with
        the type, and passes them to the logic for one argument.

        You probably don't need to override this
        """
        if cc is not None and prototype is not None:
            # Type inference for zip is broken in PyCharm 2023.* currently, so we help out manually
            loc: SimFunctionArgument
            ty: SimType
            for loc, ty in zip(cc.arg_locs(prototype), prototype.args):
                for arg in loc.get_footprint():
                    self.initialize_one_function_argument(state, func_addr, ex_loc, arg, ty)

    def initialize_one_function_argument(
        self,
        state: "ReachingDefinitionsState",
        func_addr: int,
        ex_loc: ExternalCodeLocation,
        argument_location: SimFunctionArgument,
        argument_type: Optional[SimType] = None,
    ) -> None:
        """
        This method handles the setup for _one_ argument of a function.
        This is the main method to override for custom initialization logic.

        The default implementation initializes only the argument location itself, but the signature allows
        this to support extra logic based on the _type_ of the argument as well.

        For example if the argument is a pointer to something,
        the default implementation would only set up the register with the value TOP.

        A custom implementation could instead dedicate some memory somewhere (e.g. on the heap), setup whatever object
        is being pointed to, and then put the actual pointer to this inside the register
        """
        _ = argument_type
        if isinstance(argument_location, SimRegArg):
            self._initialize_function_argument_register(state, func_addr, ex_loc, argument_location)
        elif isinstance(argument_location, SimStackArg):
            self._initialize_function_argument_stack(state, func_addr, ex_loc, argument_location)
        else:
            raise TypeError("Unsupported parameter type %s." % type(argument_location).__name__)

    def initialize_stack_pointer(
        self, state: "ReachingDefinitionsState", _func_addr: int, ex_loc: ExternalCodeLocation
    ) -> None:
        # initialize stack pointer
        sp_atom = Register(self.arch.sp_offset, self.arch.bytes)
        sp_def = Definition(sp_atom, ex_loc, tags={InitialValueTag()})
        sp = state.annotate_with_def(state._initial_stack_pointer(), sp_def)
        state.registers.store(self.arch.sp_offset, sp)

    def initialize_architectural_state(
        self,
        state: "ReachingDefinitionsState",
        func_addr: int,
        ex_loc: ExternalCodeLocation,
        rtoc_value: Optional[int] = None,
    ) -> None:
        """
        Some architectures require initialization that is specific to that architecture.

        Override this if you need to add support for an architecture that requires this, and isn't covered yet
        """
        if self.arch.name.startswith("PPC64"):
            if rtoc_value is None:
                raise TypeError("rtoc_value must be provided on PPC64.")
            offset, size = self.arch.registers["rtoc"]
            rtoc_atom = Register(offset, size)
            rtoc_def = Definition(rtoc_atom, ex_loc, tags={InitialValueTag()})
            state.all_definitions.add(rtoc_def)
            if state.analysis is not None:
                state.analysis.model.add_def(rtoc_def, ex_loc)
            rtoc = state.annotate_with_def(claripy.BVV(rtoc_value, self.arch.bits), rtoc_def)
            state.registers.store(offset, rtoc)
        elif self.arch.name.startswith("MIPS64"):
            offset, size = self.arch.registers["t9"]
            t9_atom = Register(offset, size)
            t9_def = Definition(t9_atom, ex_loc, tags={InitialValueTag()})
            state.all_definitions.add(t9_def)
            if state.analysis is not None:
                state.analysis.model.add_def(t9_def, ex_loc)
            t9 = state.annotate_with_def(claripy.BVV(func_addr, self.arch.bits), t9_def)
            state.registers.store(offset, t9)
        elif self.arch.name.startswith("MIPS"):
            if func_addr is None:
                l.warning("func_addr must not be None to initialize a function in mips")
            t9_offset = self.arch.registers["t9"][0]
            t9_atom = Register(t9_offset, self.arch.bytes)
            t9_def = Definition(t9_atom, ex_loc, tags={InitialValueTag()})
            state.all_definitions.add(t9_def)
            if state.analysis is not None:
                state.analysis.model.add_def(t9_def, ex_loc)
            t9 = state.annotate_with_def(claripy.BVV(func_addr, self.arch.bits), t9_def)
            state.registers.store(t9_offset, t9)

    def _initialize_function_argument_register(
        self,
        state: "ReachingDefinitionsState",
        func_addr: int,
        ex_loc: ExternalCodeLocation,
        arg: SimRegArg,
        value: Optional[claripy.ast.Base] = None,
    ):
        # FIXME: implement reg_offset handling in SimRegArg
        reg_offset = self.arch.registers[arg.reg_name][0]
        reg_atom = Register(reg_offset, self.arch.bytes)
        reg_def = Definition(reg_atom, ex_loc, tags={ParameterTag(function=func_addr)})
        state.all_definitions.add(reg_def)
        if state.analysis is not None:
            state.analysis.model.add_def(reg_def, ex_loc)
        if value is None:
            value = state.top(self.arch.bits)
        reg = state.annotate_with_def(value, reg_def)
        state.registers.store(reg_offset, reg)

    def _initialize_function_argument_stack(
        self, state: "ReachingDefinitionsState", func_addr: int, ex_loc: ExternalCodeLocation, arg: SimStackArg
    ):
        ml_atom = MemoryLocation(SpOffset(self.arch.bits, arg.stack_offset), arg.size)
        ml_def = Definition(ml_atom, ex_loc, tags={ParameterTag(function=func_addr)})
        state.all_definitions.add(ml_def)
        if state.analysis is not None:
            state.analysis.model.add_def(ml_def, ex_loc)
        ml = state.annotate_with_def(state.top(self.arch.bits), ml_def)
        stack_address = state.get_stack_address(state.stack_address(arg.stack_offset))
        state.stack.store(stack_address, ml, endness=self.arch.memory_endness)

    @staticmethod
    def _generate_call_string(subject: Subject, current_address: int) -> Optional[Tuple[int, ...]]:
        if isinstance(subject.content, Function):
            return (subject.content.addr,)
        elif isinstance(subject.content, CallTrace):
            if any(current_address == x.caller_func_addr for x in subject.content.callsites):
                callsites = iter(reversed([x.caller_func_addr for x in subject.content.callsites]))
                for call_addr in callsites:
                    if current_address == call_addr:
                        break
                return tuple(callsites)
            else:
                return tuple(x.caller_func_addr for x in subject.content.callsites)
        else:
            l.warning("Subject with unknown content-type")
            return None



from .successors import SimSuccessors
from .engine import SimEngine, SuccessorsMixin, TLSMixin

from .vex import HeavyVEXMixin, TrackActionsMixin, SimInspectMixin, HeavyResilienceMixin, SuperFastpathMixin
from .procedure import ProcedureMixin, ProcedureEngine
from .unicorn import SimEngineUnicorn
from .failure import SimEngineFailure
from .syscall import SimEngineSyscall
from .concrete import SimEngineConcrete
from .hook import HooksMixin
from .soot import SootMixin


# The default execution engine
# You may remove unused mixins from this default engine to speed up execution
class UberEngine(
    SimEngineFailure,
    SimEngineSyscall,
    HooksMixin,
    SimEngineUnicorn,
    SuperFastpathMixin,
    TrackActionsMixin,
    SimInspectMixin,
    HeavyResilienceMixin,
    SootMixin,
    HeavyVEXMixin,
    TLSMixin,
):
    pass


try:
    from .pcode import HeavyPcodeMixin

    class UberEnginePcode(
        SimEngineFailure, SimEngineSyscall, HooksMixin, HeavyPcodeMixin
    ):  # pylint:disable=abstract-method
        pass

except ImportError:
    pass



#!/usr/bin/env python3
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import os
import logging
import unittest

import angr
from angr.sim_variable import SimStackVariable, SimRegisterVariable
from angr.knowledge_plugins.variables import VariableType

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")

l = logging.getLogger("test_variablerecovery")


#
# Utility methods
#


class TestVariableRecovery(unittest.TestCase):
    def _compare_memory_variable(self, variable, variable_info):
        if variable_info["location"] == "stack":
            if not isinstance(variable, SimStackVariable):
                return False

            # base
            if "base" in variable_info:
                base = variable_info["base"]
                if variable.base != base:
                    return False

            # offset
            if "offset" in variable_info:
                offset = variable_info["offset"]
                if variable.offset != offset:
                    return False

            # size
            if "size" in variable_info:
                size = variable_info["size"]
                if variable.size != size:
                    return False

            return True

        else:
            if isinstance(variable, SimStackVariable):
                # it is not a variable on the stack
                return False

            raise NotImplementedError()

    def _compare_register_variable(self, variable, variable_info):  # pylint:disable=unused-argument
        if not isinstance(variable, SimRegisterVariable):
            return False

        if "reg" in variable_info:
            reg = variable_info["reg"]
            if variable.reg != reg:
                return False

        if "size" in variable_info:
            size = variable_info["size"]
            if variable.size != size:
                return False

        return True

    def _run_variable_recovery_analysis(self, func_name, groundtruth, is_fast):
        binary_path = os.path.join(test_location, "x86_64", "fauxware")
        project = angr.Project(binary_path, load_options={"auto_load_libs": False})
        cfg = project.analyses.CFG(normalize=True)
        func = cfg.kb.functions[func_name]

        # Create a temporary KnowledgeBase instance
        tmp_kb = angr.KnowledgeBase(project)

        if is_fast:
            l.debug("Running VariableRecoveryFast on function %r.", func)
            vr = project.analyses.VariableRecoveryFast(func, kb=tmp_kb)
        else:
            l.debug("Running VariableRecovery on function %r.", func)
            vr = project.analyses.VariableRecovery(func, kb=tmp_kb)

        variable_manager = vr.variable_manager[func.addr]

        for insn_addr, variables in groundtruth["variables_by_instruction"].items():
            for var_info in variables:
                var_sort = var_info["sort"]
                vars_and_offset = variable_manager.find_variables_by_insn(insn_addr, var_sort)

                # enumerate vars and find the variable that we want
                if var_sort == VariableType.MEMORY:
                    the_var = next(
                        (var for var, _ in vars_and_offset if self._compare_memory_variable(var, var_info)),
                        None,
                    )
                elif var_sort == VariableType.REGISTER:
                    the_var = next(
                        (var for var, _ in vars_and_offset if self._compare_register_variable(var, var_info)),
                        None,
                    )
                else:
                    l.error("Unsupported variable sort %s.", var_sort)
                    assert False

                assert (
                    the_var is not None
                ), "The variable {} in groundtruth at instruction {:#x} cannot be found in variable manager.".format(
                    var_info,
                    insn_addr,
                )
                l.debug("Found variable %s at %#x.", the_var, insn_addr)

        for block_addr, variables in groundtruth["phi_variables_by_block"].items():
            phi_variables = variable_manager.get_phi_variables(block_addr)
            for var_info in variables:
                var_sort = var_info["sort"]

                # enumerate vars and find the variable that we want
                if var_sort == VariableType.MEMORY:
                    the_var = next(
                        (var for var in phi_variables if self._compare_memory_variable(var, var_info)),
                        None,
                    )
                elif var_sort == VariableType.REGISTER:
                    the_var = next(
                        (var for var in phi_variables if self._compare_register_variable(var, var_info)),
                        None,
                    )
                else:
                    l.error("Unsupported variable sort %s.", var_sort)
                    assert False

                assert (
                    the_var is not None
                ), "The phi variable {} in groundtruth at block {:#x} cannot be found in variable manager.".format(
                    var_info,
                    block_addr,
                )
                l.debug("Found phi variable %s at %#x.", the_var, block_addr)

    def test_variable_recovery_fauxware_authenticate_true(self):
        self._run_variable_recovery_analysis(
            "authenticate",
            {
                "variables_by_instruction": {
                    0x40066C: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x18,
                            "size": 8,
                        },
                    ],
                    0x400670: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x20,
                            "size": 8,
                        },
                    ],
                    0x400674: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x8,
                            "size": 1,
                        },
                    ],
                    0x40067F: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x20,
                            "size": 8,
                        },
                    ],
                    0x400699: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x18,
                            "size": 8,
                        },
                    ],
                    0x4006AF: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x4,
                            "size": 4,
                        },
                    ],
                    0x4006B2: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x10,
                            "size": 1,
                        },
                    ],
                },
                "phi_variables_by_block": {
                    0x4006EB: [
                        {"sort": VariableType.REGISTER, "reg": 16, "size": 8},
                        {"sort": VariableType.REGISTER, "reg": 32, "size": 8},
                        {"sort": VariableType.REGISTER, "reg": 64, "size": 8},
                        {"sort": VariableType.REGISTER, "reg": 72, "size": 8},
                    ]
                },
            },
            True,
        )

    def test_variable_recovery_fauxware_authenticate_false(self):
        self._run_variable_recovery_analysis(
            "authenticate",
            {
                "variables_by_instruction": {
                    0x40066C: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x18,
                            "size": 8,
                        },
                    ],
                    0x400670: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x20,
                            "size": 8,
                        },
                    ],
                    0x400674: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x8,
                            "size": 1,
                        },
                    ],
                    0x40067F: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x20,
                            "size": 8,
                        },
                    ],
                    0x400699: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x18,
                            "size": 8,
                        },
                    ],
                    0x4006AF: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x4,
                            "size": 4,
                        },
                    ],
                    0x4006B2: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x10,
                            "size": 1,
                        },
                    ],
                },
                "phi_variables_by_block": {
                    0x4006EB: [
                        {"sort": VariableType.REGISTER, "reg": 16, "size": 8},
                        {"sort": VariableType.REGISTER, "reg": 32, "size": 8},
                        {"sort": VariableType.REGISTER, "reg": 64, "size": 8},
                        {"sort": VariableType.REGISTER, "reg": 72, "size": 8},
                    ]
                },
            },
            False,
        )

    def test_variable_recovery_fauxware_main_true(self):
        self._run_variable_recovery_analysis(
            "main",
            {
                "variables_by_instruction": {
                    0x400725: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x34,
                            "size": 4,
                        },
                    ],
                    0x400728: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x40,
                            "size": 8,
                        },
                    ],
                    0x40072C: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x8,
                            "size": 1,
                        },
                    ],
                    0x400730: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x18,
                            "size": 1,
                        },
                    ],
                    0x40073E: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x10,
                            "size": 1,
                        },
                    ],
                    0x400754: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x24,
                            "size": 1,
                        },
                    ],
                    0x400774: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x20,
                            "size": 1,
                        },
                    ],
                },
                "phi_variables_by_block": {},
            },
            True,
        )

    def test_variable_recovery_fauxware_main_false(self):
        self._run_variable_recovery_analysis(
            "main",
            {
                "variables_by_instruction": {
                    0x400725: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x34,
                            "size": 4,
                        },
                    ],
                    0x400728: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x40,
                            "size": 8,
                        },
                    ],
                    0x40072C: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x8,
                            "size": 1,
                        },
                    ],
                    0x400730: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x18,
                            "size": 1,
                        },
                    ],
                    0x40073E: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x10,
                            "size": 1,
                        },
                    ],
                    0x400754: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x24,
                            "size": 1,
                        },
                    ],
                    0x400774: [
                        {
                            "sort": VariableType.MEMORY,
                            "location": "stack",
                            "base": "bp",
                            "offset": -0x20,
                            "size": 1,
                        },
                    ],
                },
                "phi_variables_by_block": {},
            },
            False,
        )


if __name__ == "__main__":
    l.setLevel(logging.DEBUG)
    logging.getLogger("angr.analyses.variable_recovery_fast").setLevel(logging.DEBUG)
    logging.getLogger("angr.analyses.variable_recovery").setLevel(logging.DEBUG)

    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.sim"  # pylint:disable=redefined-builtin

import glob
import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestStateCustomization(unittest.TestCase):
    def test_stack_end(self):
        for fn in glob.glob(os.path.join(test_location, "*", "fauxware")):
            p = angr.Project(fn, auto_load_libs=False)

            # normal state
            s = p.factory.full_init_state()
            offset = s.solver.eval(p.arch.initial_sp - s.regs.sp)

            # different stack ends
            for n in [0x1337000, 0xBAAAAA00, 0x100, 0xFFFFFF00, 0x13371337000, 0xBAAAAAAA0000, 0xFFFFFFFFFFFFFF00]:
                if n.bit_length() > p.arch.bits:
                    continue
                s = p.factory.full_init_state(stack_end=n)
                assert s.solver.eval_one(s.regs.sp + offset == n)

    def test_execstack(self):
        bin_path = os.path.join(test_location, "x86_64", "fauxware")
        proj = angr.Project(bin_path, auto_load_libs=False)
        # manually mark the stack as executable
        proj.loader.main_object.execstack = True
        s = proj.factory.blank_state()
        assert s.memory._stack_perms == 7

    def test_brk(self):
        for fn in glob.glob(os.path.join(test_location, "*", "fauxware")):
            p = angr.Project(fn, auto_load_libs=False)

            # different stack ends
            for n in [0x1337000, 0xBAAAAA00, 0x100, 0xFFFFFF00, 0x13371337000, 0xBAAAAAAA0000, 0xFFFFFFFFFFFFFF00]:
                if n.bit_length() > p.arch.bits:
                    continue
                s = p.factory.full_init_state(brk=n)
                assert s.solver.eval_one(s.posix.brk == n)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestRegionIdentifier(unittest.TestCase):
    def test_smoketest(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "all"), auto_load_libs=False)
        cfg = p.analyses.CFG(normalize=True)

        main_func = cfg.kb.functions["main"]

        _ = p.analyses.RegionIdentifier(main_func)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.analyses.cfg"  # pylint:disable=redefined-builtin

import os
import unittest

from angr.project import Project

from ...common import bin_location


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestCfgManager(unittest.TestCase):
    def test_cfg_manager_copies_cfg_graphs(self):
        binary_path = os.path.join(bin_location, "tests", "x86_64", "all")
        project = Project(binary_path, auto_load_libs=False)
        _ = project.analyses.CFGFast()

        original_cfgs = project.kb.cfgs
        new_cfgs = project.kb.cfgs.copy()

        original_graph = original_cfgs.cfgs["CFGFast"].graph
        new_graph = new_cfgs.cfgs["CFGFast"].graph

        assert original_graph.edges() == new_graph.edges()
        assert original_graph.nodes() == new_graph.nodes()
        assert original_graph is not new_graph


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.state_plugins"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestMultiOpenFile(unittest.TestCase):
    def test_multi_open_file(self):
        test_bin = os.path.join(test_location, "x86_64", "test_multi_open_file")
        # auto_load_libs cannot be disabled as the test fails
        b = angr.Project(test_bin)

        pg = b.factory.simulation_manager()
        pg.active[0].options.discard("LAZY_SOLVES")
        pg.explore()

        assert len(pg.deadended) == 1

        # See the source file in binaries/tests_src/test_multi_open_file.c
        # for the tests run
        for p in pg.deadended:
            assert p.posix.dumps(2) == b""

            # Check that the temp file was deleted
            assert p.fs._files == {}

            # Check that the deleted temp file contained the appropriate string
            for event in p.history.events:
                if event.type == "fs_unlink":
                    simfile = p.fs.unlinks[event.objects["unlink_idx"]][1]
                    assert simfile.concretize() == b"foobar and baz"
                    break
            else:
                assert False


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint:disable=no-self-use
__package__ = __package__ or "tests.analyses.cfg"  # pylint:disable=redefined-builtin

import os
import unittest
import logging
import tempfile
from typing import TYPE_CHECKING, List, Sequence, Tuple

import angr
from angr.analyses import CFGFast
from angr.knowledge_plugins.cfg import MemoryData, MemoryDataSort

if TYPE_CHECKING:
    from angr.knowledge_plugins.cfg import CFGModel
    from angr.knowledge_plugins.functions import Function, FunctionManager

from ...common import bin_location


log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)
FAUXWARE_PATH = os.path.join(bin_location, "tests", "x86_64", "fauxware")


def apply_patches(proj: angr.Project, patches: List[Tuple[int, str]]):
    for addr, asm in patches:
        patch_bytes = proj.arch.keystone.asm(asm, addr, as_bytes=True)[0]
        proj.kb.patches.add_patch(addr, patch_bytes)


def assert_models_equal(model_a: "CFGModel", model_b: "CFGModel"):
    assert model_a.graph.nodes() == model_b.graph.nodes()
    assert model_a.graph.edges() == model_b.graph.edges()
    # FIXME: Check more


def assert_function_graphs_equal(function_a: "Function", function_b: "Function"):
    nodes_a = function_a.graph.nodes()
    nodes_b = function_b.graph.nodes()
    if nodes_a != nodes_b:
        log.error("Differing nodes!\nFunction:%s\nNodes A: %s\nNodes B: %s", function_a, nodes_a, nodes_b)
        assert False
    # FIXME: Check more


def assert_all_function_equal(functions_a: "FunctionManager", functions_b: "FunctionManager"):
    for f in functions_b:
        assert f in functions_a, f"Extra function: {functions_b[f]}"
    for f in functions_a:
        assert f in functions_b, f"Missing function: {functions_a[f]}"
        assert_function_graphs_equal(functions_a[f], functions_b[f])


class TestCfgCombination(unittest.TestCase):
    """
    Tests that CFGFast can run with a prior model.
    """

    def test_cfgfast_combine_with_full_model(self):
        """Run CFGFast once, then again with the model of the first."""
        proj = angr.Project(FAUXWARE_PATH, auto_load_libs=False)
        cfg_a = proj.analyses[CFGFast].prep()()
        cfg_b = proj.analyses[CFGFast].prep()(model=cfg_a.model.copy())
        assert_models_equal(cfg_a.model, cfg_b.model)
        assert_all_function_equal(cfg_a.functions, cfg_b.functions)

    def test_cfgfast_combine_with_partial_model(self):
        """Run CFGFast on a region, then again on a second region with the model of the first."""

        # Initial analysis just to pick up expected addresses
        proj = angr.Project(FAUXWARE_PATH, auto_load_libs=False)
        cfg = proj.analyses[CFGFast].prep()()
        accepted_addr = cfg.functions["accepted"].addr
        rejected_addr = cfg.functions["rejected"].addr
        accepted_regions = [(n.addr, n.addr + n.size) for n in cfg.functions["accepted"].nodes]
        rejected_regions = [(n.addr, n.addr + n.size) for n in cfg.functions["rejected"].nodes]

        # Run partial analysis on the nodes we care about
        proj = angr.Project(FAUXWARE_PATH, auto_load_libs=False)
        cfg = proj.analyses[CFGFast].prep()(regions=accepted_regions)
        assert accepted_addr in cfg.functions.function_addrs_set
        assert rejected_addr not in cfg.functions.function_addrs_set

        # Check continued analysis over another region combines correctly
        cfg = proj.analyses[CFGFast].prep()(regions=rejected_regions, model=cfg.model.copy())
        assert accepted_addr in cfg.functions.function_addrs_set
        assert rejected_addr in cfg.functions.function_addrs_set

        # Check analysis over union of regions yields same result
        proj = angr.Project(FAUXWARE_PATH, auto_load_libs=False)
        regions = accepted_regions + rejected_regions
        cfg_combined = proj.analyses[CFGFast].prep()(regions=regions)
        assert_models_equal(cfg.model, cfg_combined.model)
        assert_all_function_equal(cfg.functions, cfg_combined.functions)

    # FIXME: Add test of first analyzing a function A, then analyzing a function B called by A


class TestCfgReclassification(unittest.TestCase):
    """
    Tests that code/data can be reclassified in CFG.
    """

    def test_cfgfast_preclassify_code_as_data(self):
        """Classify a code region as data in an empty model, run CFGFast, ensure region remains classified as data."""
        proj = angr.Project(FAUXWARE_PATH, auto_load_libs=False)
        model = proj.kb.cfgs.new_model("initial")
        md = MemoryData(0x40069D, 22, MemoryDataSort.String)
        model.memory_data[md.addr] = md.copy()
        cfg = proj.analyses[CFGFast].prep()(model=model)
        assert cfg.model.memory_data[md.addr] == md

        # Make sure the memory data item is not covered by any node
        assert len(cfg.functions["authenticate"].graph.nodes) == 5
        for n in cfg.functions["authenticate"].graph.nodes:
            assert md.addr >= (n.addr + n.size) or n.addr >= (md.addr + md.size)
            if (n.addr + n.size) == md.addr:
                log.debug("Found adjacent node %s", n)
                # FIXME: Successor address is still the same

        # FIXME: Also test at function boundary

    def test_cfgfast_reclassify_code_as_data(self):
        """Run CFGFast, re-classify code within a function as data, then re-flow the function."""
        proj = angr.Project(FAUXWARE_PATH, auto_load_libs=False)

        # Initial analysis
        cfg = proj.analyses[CFGFast].prep()()
        func = cfg.functions["authenticate"]
        function_addr = func.addr
        original_number_of_nodes = len(func.graph.nodes)

        # Define the data region and run analysis for function reconstruction
        md = MemoryData(0x40069D, 22, MemoryDataSort.String)
        cfg.model.memory_data[md.addr] = md.copy()
        cfg.model.clear_region_for_reflow(func.addr)
        cfg = proj.analyses[CFGFast].prep()(
            symbols=False,
            function_prologues=False,
            start_at_entry=False,
            force_smart_scan=False,
            force_complete_scan=False,
            function_starts=[function_addr],
            model=cfg.model,
        )

        # Check memory data remains as configured and is not overlapped by any node
        assert cfg.model.memory_data[md.addr] == md
        assert len(cfg.functions["authenticate"].graph.nodes) == 5
        for n in cfg.functions["authenticate"].graph.nodes:
            assert md.addr >= (n.addr + n.size) or n.addr >= (md.addr + md.size)

        # Now re-define the data as code again and ensure we have the correct number of nodes
        del cfg.model.memory_data[md.addr]
        cfg.model.clear_region_for_reflow(func.addr)
        cfg = proj.analyses[CFGFast].prep()(
            symbols=False,
            function_prologues=False,
            start_at_entry=False,
            force_smart_scan=False,
            force_complete_scan=False,
            function_starts=[function_addr],
            model=cfg.model,
        )

        assert len(cfg.functions["authenticate"].graph.nodes) == original_number_of_nodes

        # FIXME: Test at function boundary

    # FIXME: Test at partial offset into some data item
    def test_cfgfast_preclassify_data_as_code(self):
        """Classify some code that would not normally be classified as code."""
        code = """
        _start:
            ret

        not_discovered:
            xor rax, rax
            mov rcx, 5
            .here:
            inc rax
            dec rcx
            jnz .here
            ret
        """

        not_discovered_addr = 0x1
        proj = angr.load_shellcode(code, "AMD64")
        cfg = proj.analyses[CFGFast].prep()(force_smart_scan=False)
        assert len(cfg.functions) == 1

        proj = angr.load_shellcode(code, "AMD64")
        cfg = proj.analyses[CFGFast].prep()(force_smart_scan=False, function_starts=[not_discovered_addr])
        assert len(cfg.functions) == 2
        assert len(cfg.functions[not_discovered_addr].block_addrs) == 3

    def test_cfgfast_reclassify_data_as_code(self):
        """Run CFGFast, then re-classify some assumed data as code and run again."""
        code = """
        _start:
            mov rax, [not_discovered]
            ret

        not_discovered:
            xor rax, rax
            mov rcx, 5
            .here:
            inc rax
            dec rcx
            jnz .here
            ret
        """

        proj = angr.load_shellcode(code, "AMD64")
        cfg = proj.analyses[CFGFast].prep()(force_smart_scan=False)
        assert len(cfg.functions) == 1
        not_discovered_addr = 0xB
        del cfg.model.memory_data[not_discovered_addr]
        cfg = proj.analyses[CFGFast].prep()(
            start_at_entry=False, force_smart_scan=False, function_starts=[not_discovered_addr], model=cfg.model
        )
        assert len(cfg.functions) == 2
        assert len(cfg.functions[0xB].block_addrs) == 3


class TestCfgPatching(unittest.TestCase):
    """
    Test that patches made to the binary are correctly reflected in CFG.
    """

    def _test_patch(self, patches: Sequence[Tuple[int, str]]):
        unpatched_binary_path = FAUXWARE_PATH
        common_cfg_options = {
            "normalize": True,
            "resolve_indirect_jumps": True,
            "data_references": True,
            # "force_smart_scan": False,
        }

        # Create and load a pre-patched binary
        log.debug("Recovering pre-patched CFG")
        proj = angr.Project(unpatched_binary_path, auto_load_libs=False)
        apply_patches(proj, patches)

        with tempfile.NamedTemporaryFile(prefix="fauxware-patched-", delete=False) as f:
            f.write(proj.kb.patches.apply_patches_to_binary())
            f.close()
            prepatched_proj = angr.Project(f.name, auto_load_libs=False)
            expected_cfg = prepatched_proj.analyses[CFGFast].prep()(**common_cfg_options)

            # Now create a new project, recover CFG, then patch and recover CFG again
            proj = angr.Project(unpatched_binary_path, auto_load_libs=False)
            log.debug("Recovering CFG before patching")
            cfg_before_patching = proj.analyses[CFGFast].prep()(**common_cfg_options)
            apply_patches(proj, patches)

            log.debug("Recovering CFG after patching")
            for p in proj.kb.patches.values():
                cfg_before_patching.model.clear_region_for_reflow(p.addr, len(p.new_bytes))
            cfg_after_patching = proj.analyses[CFGFast].prep()(**common_cfg_options, model=cfg_before_patching.model)

            # Verify that the CFG of the patched binary matches the CFG of the pre-patched binary
            assert_models_equal(expected_cfg.model, cfg_after_patching.model)

            os.unlink(f.name)

    #
    # Patches that do not change block or function size
    #

    def test_cfg_patch_const_operand(self):
        """
        Patch a block, changing a data reference, with no effect on control.

        Change print of "Username: " to "Password: ".
        """
        self._test_patch([(0x400734, "mov edi, 0x400920")])

    def test_cfg_patch_ret_value(self):
        """
        Patch a block to redirect control, without changing the graph.

        Change return value of `authenticate` in rejection branch to 1.
        """
        self._test_patch([(0x4006E6, "mov eax, 1")])

    def test_cfg_patch_branch(self):
        """
        Patch a block, changing the graph.

        Patch `authenticate` to always jump to accept branch, eliminating 1 block.
        """
        self._test_patch([(0x4006DD, "jne 0x4006df")])

    def test_cfg_patch_call_target(self):
        """
        Patch a block, changing the graph, eliminate a cross reference.

        Change call of `rejected` to `accepted`.
        """
        self._test_patch([(0x4007CE, "call 0x4006ed")])

    # FIXME: Patches that change indirect jumps

    #
    # Patches that shrink blocks/function
    #

    def test_cfg_patch_shrink_encoding(self):
        """
        Shorten a block, but do not change graph.

        Use a shorter instruction encoding.
        """
        self._test_patch([(0x4006DF, "xor eax, eax;  inc eax;  jmp 0x4006eb")])

    def test_cfg_patch_shrink_branch(self):
        """
        Shorten a block, changing the graph.

        Remove `strcmp` check in `authenticate`, just jump to accept branch.
        """
        self._test_patch([(0x4006DB, "jmp 0x4006df")])

    def test_cfg_patch_shrink_ret_value(self):
        """
        Shorten a block, truncating a function.

        Patch `authenticate` to always return 1
        """
        self._test_patch([(0x400664, "xor rax, rax;  inc rax;  ret")])

    #
    # Patches that grow blocks
    #

    def test_cfg_patch_grow_block_fallthru(self):
        """
        Patch a block to cover another block.

        Ignore return value of `authenticate`, fallthru to accept branch.
        """
        self._test_patch([(0x4007BB, "nop;  nop;")])

    def test_cfg_patch_grow_nocall(self):
        """
        Patch a block to eliminate all cross references to a function.

        Patch out call to `authenticate`, fall thru.
        """
        self._test_patch([(0x4007AE, "xor rax, rax;  nop;  nop")])

    def test_cfg_patch_grow_into_inter_function_padding(self):
        """
        Patch a block to grow into padded space between functions.

        Prepend several NOPs to the last block of `main`.
        """
        self._test_patch([(0x4007D3, "nop; nop; nop; nop; leave; ret")])

    def test_cfg_patch_grow_function_fallthru(self):
        """
        Patch a block that extends into another function.

        Cut off the end of `authenticate` so it falls into `accepted`.
        """
        self._test_patch([(0x4006EC, "nop")])
        # Will we have two functions? accepted() is still called so it will probably mark a function


if __name__ == "__main__":
    logging.basicConfig()
    log.setLevel(logging.DEBUG)
    logging.getLogger("angr.analyses.cfg.cfg_fast").setLevel(logging.DEBUG)
    logging.getLogger("angr.analyses.cfg.cfg_base").setLevel(logging.DEBUG)
    unittest.main()



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("dhcpcsvc6.dll")
prototypes = \
    {
        #
        'Dhcpv6CApiInitialize': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Version"]),
        #
        'Dhcpv6CApiCleanup': SimTypeFunction([], SimTypeBottom(label="Void")),
        #
        'Dhcpv6RequestParams': SimTypeFunction([SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"Flags": SimTypeInt(signed=False, label="UInt32"), "Data": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "nBytesData": SimTypeInt(signed=False, label="UInt32")}, name="DHCPV6CAPI_CLASSID", pack=False, align=None), offset=0), SimStruct({"nParams": SimTypeInt(signed=False, label="UInt32"), "Params": SimTypePointer(SimStruct({"Flags": SimTypeInt(signed=False, label="UInt32"), "OptionId": SimTypeInt(signed=False, label="UInt32"), "IsVendor": SimTypeInt(signed=True, label="Int32"), "Data": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "nBytesData": SimTypeInt(signed=False, label="UInt32")}, name="DHCPV6CAPI_PARAMS", pack=False, align=None), offset=0)}, name="DHCPV6CAPI_PARAMS_ARRAY", pack=False, align=None), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["forceNewInform", "reserved", "adapterName", "classId", "recdParams", "buffer", "pSize"]),
        #
        'Dhcpv6RequestPrefix': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"Flags": SimTypeInt(signed=False, label="UInt32"), "Data": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "nBytesData": SimTypeInt(signed=False, label="UInt32")}, name="DHCPV6CAPI_CLASSID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"nPrefixes": SimTypeInt(signed=False, label="UInt32"), "prefixArray": SimTypePointer(SimStruct({"prefix": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeInt(signed=False, label="UInt32"), "preferredLifeTime": SimTypeInt(signed=False, label="UInt32"), "validLifeTime": SimTypeInt(signed=False, label="UInt32"), "status": SimTypeInt(signed=False, label="StatusCode")}, name="DHCPV6Prefix", pack=False, align=None), offset=0), "iaid": SimTypeInt(signed=False, label="UInt32"), "T1": SimTypeLongLong(signed=True, label="Int64"), "T2": SimTypeLongLong(signed=True, label="Int64"), "MaxLeaseExpirationTime": SimTypeLongLong(signed=True, label="Int64"), "LastRenewalTime": SimTypeLongLong(signed=True, label="Int64"), "status": SimTypeInt(signed=False, label="StatusCode"), "ServerId": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "ServerIdLen": SimTypeInt(signed=False, label="UInt32")}, name="DHCPV6PrefixLeaseInformation", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["adapterName", "pclassId", "prefixleaseInfo", "pdwTimeToWait"]),
        #
        'Dhcpv6RenewPrefix': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"Flags": SimTypeInt(signed=False, label="UInt32"), "Data": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "nBytesData": SimTypeInt(signed=False, label="UInt32")}, name="DHCPV6CAPI_CLASSID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"nPrefixes": SimTypeInt(signed=False, label="UInt32"), "prefixArray": SimTypePointer(SimStruct({"prefix": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeInt(signed=False, label="UInt32"), "preferredLifeTime": SimTypeInt(signed=False, label="UInt32"), "validLifeTime": SimTypeInt(signed=False, label="UInt32"), "status": SimTypeInt(signed=False, label="StatusCode")}, name="DHCPV6Prefix", pack=False, align=None), offset=0), "iaid": SimTypeInt(signed=False, label="UInt32"), "T1": SimTypeLongLong(signed=True, label="Int64"), "T2": SimTypeLongLong(signed=True, label="Int64"), "MaxLeaseExpirationTime": SimTypeLongLong(signed=True, label="Int64"), "LastRenewalTime": SimTypeLongLong(signed=True, label="Int64"), "status": SimTypeInt(signed=False, label="StatusCode"), "ServerId": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "ServerIdLen": SimTypeInt(signed=False, label="UInt32")}, name="DHCPV6PrefixLeaseInformation", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["adapterName", "pclassId", "prefixleaseInfo", "pdwTimeToWait", "bValidatePrefix"]),
        #
        'Dhcpv6ReleasePrefix': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"Flags": SimTypeInt(signed=False, label="UInt32"), "Data": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "nBytesData": SimTypeInt(signed=False, label="UInt32")}, name="DHCPV6CAPI_CLASSID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"nPrefixes": SimTypeInt(signed=False, label="UInt32"), "prefixArray": SimTypePointer(SimStruct({"prefix": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 16), "prefixLength": SimTypeInt(signed=False, label="UInt32"), "preferredLifeTime": SimTypeInt(signed=False, label="UInt32"), "validLifeTime": SimTypeInt(signed=False, label="UInt32"), "status": SimTypeInt(signed=False, label="StatusCode")}, name="DHCPV6Prefix", pack=False, align=None), offset=0), "iaid": SimTypeInt(signed=False, label="UInt32"), "T1": SimTypeLongLong(signed=True, label="Int64"), "T2": SimTypeLongLong(signed=True, label="Int64"), "MaxLeaseExpirationTime": SimTypeLongLong(signed=True, label="Int64"), "LastRenewalTime": SimTypeLongLong(signed=True, label="Int64"), "status": SimTypeInt(signed=False, label="StatusCode"), "ServerId": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "ServerIdLen": SimTypeInt(signed=False, label="UInt32")}, name="DHCPV6PrefixLeaseInformation", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["adapterName", "classId", "leaseInfo"]),
    }

lib.set_prototypes(prototypes)



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long

import unittest

import angr


class TestSimTime(unittest.TestCase):
    def test_gettimeofday(self):
        proc = angr.SIM_PROCEDURES["posix"]["gettimeofday"]()

        s = angr.SimState(arch="amd64")
        s.regs.rdi = 0x8000
        s.regs.rsi = 0

        s.options.add(angr.options.USE_SYSTEM_TIMES)
        proc.execute(s)
        assert not s.mem[0x8000].qword.resolved.symbolic
        assert not s.mem[0x8008].qword.resolved.symbolic

        s.options.discard(angr.options.USE_SYSTEM_TIMES)
        proc.execute(s)
        assert s.mem[0x8000].qword.resolved.symbolic
        assert s.mem[0x8008].qword.resolved.symbolic

    def test_clock_gettime(self):
        proc = angr.SIM_PROCEDURES["posix"]["clock_gettime"]()

        s = angr.SimState(arch="amd64")
        s.regs.rdi = 0
        s.regs.rsi = 0x8000

        s.options.add(angr.options.USE_SYSTEM_TIMES)
        proc.execute(s)
        assert not s.mem[0x8000].qword.resolved.symbolic
        assert not s.mem[0x8008].qword.resolved.symbolic

        s.options.discard(angr.options.USE_SYSTEM_TIMES)
        proc.execute(s)
        assert s.mem[0x8000].qword.resolved.symbolic
        assert s.mem[0x8008].qword.resolved.symbolic


if __name__ == "__main__":
    unittest.main()



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("activeds.dll")
prototypes = \
    {
        #
        'ADsGetObject': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszPathName", "riid", "ppObject"]),
        #
        'ADsBuildEnumerator': SimTypeFunction([SimTypeBottom(label="IADsContainer"), SimTypePointer(SimTypeBottom(label="IEnumVARIANT"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pADsContainer", "ppEnumVariant"]),
        #
        'ADsFreeEnumerator': SimTypeFunction([SimTypeBottom(label="IEnumVARIANT")], SimTypeInt(signed=True, label="Int32"), arg_names=["pEnumVariant"]),
        #
        'ADsEnumerateNext': SimTypeFunction([SimTypeBottom(label="IEnumVARIANT"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"vt": SimTypeShort(signed=False, label="UInt16"), "wReserved1": SimTypeShort(signed=False, label="UInt16"), "wReserved2": SimTypeShort(signed=False, label="UInt16"), "wReserved3": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimUnion({"llVal": SimTypeLongLong(signed=True, label="Int64"), "lVal": SimTypeInt(signed=True, label="Int32"), "bVal": SimTypeChar(label="Byte"), "iVal": SimTypeShort(signed=True, label="Int16"), "fltVal": SimTypeFloat(size=32), "dblVal": SimTypeFloat(size=64), "boolVal": SimTypeShort(signed=True, label="Int16"), "__OBSOLETE__VARIANT_BOOL": SimTypeShort(signed=True, label="Int16"), "scode": SimTypeInt(signed=True, label="Int32"), "cyVal": SimTypeBottom(label="CY"), "date": SimTypeFloat(size=64), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "punkVal": SimTypeBottom(label="IUnknown"), "pdispVal": SimTypeBottom(label="IDispatch"), "parray": SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), "pbVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "piVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "plVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pllVal": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pfltVal": SimTypePointer(SimTypeFloat(size=32), offset=0), "pdblVal": SimTypePointer(SimTypeFloat(size=64), offset=0), "pboolVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "__OBSOLETE__VARIANT_PBOOL": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "pscode": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pcyVal": SimTypePointer(SimTypeBottom(label="CY"), offset=0), "pdate": SimTypePointer(SimTypeFloat(size=64), offset=0), "pbstrVal": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "ppunkVal": SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0), "ppdispVal": SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0), "pparray": SimTypePointer(SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), offset=0), "pvarVal": SimTypePointer(SimTypeBottom(label="VARIANT"), offset=0), "byref": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cVal": SimTypeBottom(label="CHAR"), "uiVal": SimTypeShort(signed=False, label="UInt16"), "ulVal": SimTypeInt(signed=False, label="UInt32"), "ullVal": SimTypeLongLong(signed=False, label="UInt64"), "intVal": SimTypeInt(signed=True, label="Int32"), "uintVal": SimTypeInt(signed=False, label="UInt32"), "pdecVal": SimTypePointer(SimTypeBottom(label="DECIMAL"), offset=0), "pcVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "puiVal": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "pulVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pullVal": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "pintVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "puintVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "Anonymous": SimStruct({"pvRecord": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "pRecInfo": SimTypeBottom(label="IRecordInfo")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="_Anonymous_e__Struct", pack=False, align=None), "decVal": SimTypeBottom(label="DECIMAL")}, name="<anon>", label="None")}, name="VARIANT", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pEnumVariant", "cElements", "pvar", "pcElementsFetched"]),
        #
        'ADsBuildVarArrayStr': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"vt": SimTypeShort(signed=False, label="UInt16"), "wReserved1": SimTypeShort(signed=False, label="UInt16"), "wReserved2": SimTypeShort(signed=False, label="UInt16"), "wReserved3": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimUnion({"llVal": SimTypeLongLong(signed=True, label="Int64"), "lVal": SimTypeInt(signed=True, label="Int32"), "bVal": SimTypeChar(label="Byte"), "iVal": SimTypeShort(signed=True, label="Int16"), "fltVal": SimTypeFloat(size=32), "dblVal": SimTypeFloat(size=64), "boolVal": SimTypeShort(signed=True, label="Int16"), "__OBSOLETE__VARIANT_BOOL": SimTypeShort(signed=True, label="Int16"), "scode": SimTypeInt(signed=True, label="Int32"), "cyVal": SimTypeBottom(label="CY"), "date": SimTypeFloat(size=64), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "punkVal": SimTypeBottom(label="IUnknown"), "pdispVal": SimTypeBottom(label="IDispatch"), "parray": SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), "pbVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "piVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "plVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pllVal": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pfltVal": SimTypePointer(SimTypeFloat(size=32), offset=0), "pdblVal": SimTypePointer(SimTypeFloat(size=64), offset=0), "pboolVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "__OBSOLETE__VARIANT_PBOOL": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "pscode": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pcyVal": SimTypePointer(SimTypeBottom(label="CY"), offset=0), "pdate": SimTypePointer(SimTypeFloat(size=64), offset=0), "pbstrVal": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "ppunkVal": SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0), "ppdispVal": SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0), "pparray": SimTypePointer(SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), offset=0), "pvarVal": SimTypePointer(SimTypeBottom(label="VARIANT"), offset=0), "byref": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cVal": SimTypeBottom(label="CHAR"), "uiVal": SimTypeShort(signed=False, label="UInt16"), "ulVal": SimTypeInt(signed=False, label="UInt32"), "ullVal": SimTypeLongLong(signed=False, label="UInt64"), "intVal": SimTypeInt(signed=True, label="Int32"), "uintVal": SimTypeInt(signed=False, label="UInt32"), "pdecVal": SimTypePointer(SimTypeBottom(label="DECIMAL"), offset=0), "pcVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "puiVal": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "pulVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pullVal": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "pintVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "puintVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "Anonymous": SimStruct({"pvRecord": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "pRecInfo": SimTypeBottom(label="IRecordInfo")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="_Anonymous_e__Struct", pack=False, align=None), "decVal": SimTypeBottom(label="DECIMAL")}, name="<anon>", label="None")}, name="VARIANT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lppPathNames", "dwPathNames", "pVar"]),
        #
        'ADsBuildVarArrayInt': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"vt": SimTypeShort(signed=False, label="UInt16"), "wReserved1": SimTypeShort(signed=False, label="UInt16"), "wReserved2": SimTypeShort(signed=False, label="UInt16"), "wReserved3": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimUnion({"llVal": SimTypeLongLong(signed=True, label="Int64"), "lVal": SimTypeInt(signed=True, label="Int32"), "bVal": SimTypeChar(label="Byte"), "iVal": SimTypeShort(signed=True, label="Int16"), "fltVal": SimTypeFloat(size=32), "dblVal": SimTypeFloat(size=64), "boolVal": SimTypeShort(signed=True, label="Int16"), "__OBSOLETE__VARIANT_BOOL": SimTypeShort(signed=True, label="Int16"), "scode": SimTypeInt(signed=True, label="Int32"), "cyVal": SimTypeBottom(label="CY"), "date": SimTypeFloat(size=64), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "punkVal": SimTypeBottom(label="IUnknown"), "pdispVal": SimTypeBottom(label="IDispatch"), "parray": SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), "pbVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "piVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "plVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pllVal": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pfltVal": SimTypePointer(SimTypeFloat(size=32), offset=0), "pdblVal": SimTypePointer(SimTypeFloat(size=64), offset=0), "pboolVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "__OBSOLETE__VARIANT_PBOOL": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "pscode": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pcyVal": SimTypePointer(SimTypeBottom(label="CY"), offset=0), "pdate": SimTypePointer(SimTypeFloat(size=64), offset=0), "pbstrVal": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "ppunkVal": SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0), "ppdispVal": SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0), "pparray": SimTypePointer(SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), offset=0), "pvarVal": SimTypePointer(SimTypeBottom(label="VARIANT"), offset=0), "byref": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cVal": SimTypeBottom(label="CHAR"), "uiVal": SimTypeShort(signed=False, label="UInt16"), "ulVal": SimTypeInt(signed=False, label="UInt32"), "ullVal": SimTypeLongLong(signed=False, label="UInt64"), "intVal": SimTypeInt(signed=True, label="Int32"), "uintVal": SimTypeInt(signed=False, label="UInt32"), "pdecVal": SimTypePointer(SimTypeBottom(label="DECIMAL"), offset=0), "pcVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "puiVal": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "pulVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pullVal": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "pintVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "puintVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "Anonymous": SimStruct({"pvRecord": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "pRecInfo": SimTypeBottom(label="IRecordInfo")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="_Anonymous_e__Struct", pack=False, align=None), "decVal": SimTypeBottom(label="DECIMAL")}, name="<anon>", label="None")}, name="VARIANT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpdwObjectTypes", "dwObjectTypes", "pVar"]),
        #
        'ADsOpenObject': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="ADS_AUTHENTICATION_ENUM"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpszPathName", "lpszUserName", "lpszPassword", "dwReserved", "riid", "ppObject"]),
        #
        'ADsGetLastError': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpError", "lpErrorBuf", "dwErrorBufLen", "lpNameBuf", "dwNameBufLen"]),
        #
        'ADsSetLastError': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["dwErr", "pszError", "pszProvider"]),
        #
        'AllocADsMem': SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cb"]),
        #
        'FreeADsMem': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pMem"]),
        #
        'ReallocADsMem': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["pOldMem", "cbOld", "cbNew"]),
        #
        'AllocADsStr': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypePointer(SimTypeChar(label="Char"), offset=0), arg_names=["pStr"]),
        #
        'FreeADsStr': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pStr"]),
        #
        'ReallocADsStr': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ppStr", "pStr"]),
        #
        'ADsEncodeBinaryData': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pbSrcData", "dwSrcLen", "ppszDestData"]),
        #
        'ADsDecodeBinaryData': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["szSrcData", "ppbDestData", "pdwDestLen"]),
        #
        'PropVariantToAdsType': SimTypeFunction([SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"vt": SimTypeShort(signed=False, label="UInt16"), "wReserved1": SimTypeShort(signed=False, label="UInt16"), "wReserved2": SimTypeShort(signed=False, label="UInt16"), "wReserved3": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimUnion({"llVal": SimTypeLongLong(signed=True, label="Int64"), "lVal": SimTypeInt(signed=True, label="Int32"), "bVal": SimTypeChar(label="Byte"), "iVal": SimTypeShort(signed=True, label="Int16"), "fltVal": SimTypeFloat(size=32), "dblVal": SimTypeFloat(size=64), "boolVal": SimTypeShort(signed=True, label="Int16"), "__OBSOLETE__VARIANT_BOOL": SimTypeShort(signed=True, label="Int16"), "scode": SimTypeInt(signed=True, label="Int32"), "cyVal": SimTypeBottom(label="CY"), "date": SimTypeFloat(size=64), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "punkVal": SimTypeBottom(label="IUnknown"), "pdispVal": SimTypeBottom(label="IDispatch"), "parray": SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), "pbVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "piVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "plVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pllVal": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pfltVal": SimTypePointer(SimTypeFloat(size=32), offset=0), "pdblVal": SimTypePointer(SimTypeFloat(size=64), offset=0), "pboolVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "__OBSOLETE__VARIANT_PBOOL": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "pscode": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pcyVal": SimTypePointer(SimTypeBottom(label="CY"), offset=0), "pdate": SimTypePointer(SimTypeFloat(size=64), offset=0), "pbstrVal": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "ppunkVal": SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0), "ppdispVal": SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0), "pparray": SimTypePointer(SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), offset=0), "pvarVal": SimTypePointer(SimTypeBottom(label="VARIANT"), offset=0), "byref": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cVal": SimTypeBottom(label="CHAR"), "uiVal": SimTypeShort(signed=False, label="UInt16"), "ulVal": SimTypeInt(signed=False, label="UInt32"), "ullVal": SimTypeLongLong(signed=False, label="UInt64"), "intVal": SimTypeInt(signed=True, label="Int32"), "uintVal": SimTypeInt(signed=False, label="UInt32"), "pdecVal": SimTypePointer(SimTypeBottom(label="DECIMAL"), offset=0), "pcVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "puiVal": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "pulVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pullVal": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "pintVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "puintVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "Anonymous": SimStruct({"pvRecord": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "pRecInfo": SimTypeBottom(label="IRecordInfo")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="_Anonymous_e__Struct", pack=False, align=None), "decVal": SimTypeBottom(label="DECIMAL")}, name="<anon>", label="None")}, name="VARIANT", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimStruct({"dwType": SimTypeInt(signed=False, label="ADSTYPEENUM"), "Anonymous": SimUnion({"DNString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "CaseExactString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "CaseIgnoreString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "PrintableString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "NumericString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "Boolean": SimTypeInt(signed=False, label="UInt32"), "Integer": SimTypeInt(signed=False, label="UInt32"), "OctetString": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_OCTET_STRING", pack=False, align=None), "UTCTime": SimStruct({"wYear": SimTypeShort(signed=False, label="UInt16"), "wMonth": SimTypeShort(signed=False, label="UInt16"), "wDayOfWeek": SimTypeShort(signed=False, label="UInt16"), "wDay": SimTypeShort(signed=False, label="UInt16"), "wHour": SimTypeShort(signed=False, label="UInt16"), "wMinute": SimTypeShort(signed=False, label="UInt16"), "wSecond": SimTypeShort(signed=False, label="UInt16"), "wMilliseconds": SimTypeShort(signed=False, label="UInt16")}, name="SYSTEMTIME", pack=False, align=None), "LargeInteger": SimTypeBottom(label="LARGE_INTEGER"), "ClassName": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "ProviderSpecific": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_PROV_SPECIFIC", pack=False, align=None), "pCaseIgnoreList": SimTypePointer(SimStruct({"Next": SimTypePointer(SimTypeBottom(label="ADS_CASEIGNORE_LIST"), offset=0), "String": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_CASEIGNORE_LIST", pack=False, align=None), offset=0), "pOctetList": SimTypePointer(SimStruct({"Next": SimTypePointer(SimTypeBottom(label="ADS_OCTET_LIST"), offset=0), "Length": SimTypeInt(signed=False, label="UInt32"), "Data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_OCTET_LIST", pack=False, align=None), offset=0), "pPath": SimTypePointer(SimStruct({"Type": SimTypeInt(signed=False, label="UInt32"), "VolumeName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Path": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_PATH", pack=False, align=None), offset=0), "pPostalAddress": SimTypePointer(SimStruct({"PostalAddress": SimTypeFixedSizeArray(SimTypePointer(SimTypeChar(label="Char"), offset=0), 6)}, name="ADS_POSTALADDRESS", pack=False, align=None), offset=0), "Timestamp": SimStruct({"WholeSeconds": SimTypeInt(signed=False, label="UInt32"), "EventID": SimTypeInt(signed=False, label="UInt32")}, name="ADS_TIMESTAMP", pack=False, align=None), "BackLink": SimStruct({"RemoteID": SimTypeInt(signed=False, label="UInt32"), "ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_BACKLINK", pack=False, align=None), "pTypedName": SimTypePointer(SimStruct({"ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Level": SimTypeInt(signed=False, label="UInt32"), "Interval": SimTypeInt(signed=False, label="UInt32")}, name="ADS_TYPEDNAME", pack=False, align=None), offset=0), "Hold": SimStruct({"ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Amount": SimTypeInt(signed=False, label="UInt32")}, name="ADS_HOLD", pack=False, align=None), "pNetAddress": SimTypePointer(SimStruct({"AddressType": SimTypeInt(signed=False, label="UInt32"), "AddressLength": SimTypeInt(signed=False, label="UInt32"), "Address": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NETADDRESS", pack=False, align=None), offset=0), "pReplicaPointer": SimTypePointer(SimStruct({"ServerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ReplicaType": SimTypeInt(signed=False, label="UInt32"), "ReplicaNumber": SimTypeInt(signed=False, label="UInt32"), "Count": SimTypeInt(signed=False, label="UInt32"), "ReplicaAddressHints": SimTypePointer(SimStruct({"AddressType": SimTypeInt(signed=False, label="UInt32"), "AddressLength": SimTypeInt(signed=False, label="UInt32"), "Address": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NETADDRESS", pack=False, align=None), offset=0)}, name="ADS_REPLICAPOINTER", pack=False, align=None), offset=0), "pFaxNumber": SimTypePointer(SimStruct({"TelephoneNumber": SimTypePointer(SimTypeChar(label="Char"), offset=0), "NumberOfBits": SimTypeInt(signed=False, label="UInt32"), "Parameters": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_FAXNUMBER", pack=False, align=None), offset=0), "Email": SimStruct({"Address": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Type": SimTypeInt(signed=False, label="UInt32")}, name="ADS_EMAIL", pack=False, align=None), "SecurityDescriptor": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NT_SECURITY_DESCRIPTOR", pack=False, align=None), "pDNWithBinary": SimTypePointer(SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpBinaryValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pszDNString": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_DN_WITH_BINARY", pack=False, align=None), offset=0), "pDNWithString": SimTypePointer(SimStruct({"pszStringValue": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszDNString": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_DN_WITH_STRING", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="ADSVALUE", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pVariant", "dwNumVariant", "ppAdsValues", "pdwNumValues"]),
        #
        'AdsTypeToPropVariant': SimTypeFunction([SimTypePointer(SimStruct({"dwType": SimTypeInt(signed=False, label="ADSTYPEENUM"), "Anonymous": SimUnion({"DNString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "CaseExactString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "CaseIgnoreString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "PrintableString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "NumericString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "Boolean": SimTypeInt(signed=False, label="UInt32"), "Integer": SimTypeInt(signed=False, label="UInt32"), "OctetString": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_OCTET_STRING", pack=False, align=None), "UTCTime": SimStruct({"wYear": SimTypeShort(signed=False, label="UInt16"), "wMonth": SimTypeShort(signed=False, label="UInt16"), "wDayOfWeek": SimTypeShort(signed=False, label="UInt16"), "wDay": SimTypeShort(signed=False, label="UInt16"), "wHour": SimTypeShort(signed=False, label="UInt16"), "wMinute": SimTypeShort(signed=False, label="UInt16"), "wSecond": SimTypeShort(signed=False, label="UInt16"), "wMilliseconds": SimTypeShort(signed=False, label="UInt16")}, name="SYSTEMTIME", pack=False, align=None), "LargeInteger": SimTypeBottom(label="LARGE_INTEGER"), "ClassName": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "ProviderSpecific": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_PROV_SPECIFIC", pack=False, align=None), "pCaseIgnoreList": SimTypePointer(SimStruct({"Next": SimTypePointer(SimTypeBottom(label="ADS_CASEIGNORE_LIST"), offset=0), "String": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_CASEIGNORE_LIST", pack=False, align=None), offset=0), "pOctetList": SimTypePointer(SimStruct({"Next": SimTypePointer(SimTypeBottom(label="ADS_OCTET_LIST"), offset=0), "Length": SimTypeInt(signed=False, label="UInt32"), "Data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_OCTET_LIST", pack=False, align=None), offset=0), "pPath": SimTypePointer(SimStruct({"Type": SimTypeInt(signed=False, label="UInt32"), "VolumeName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Path": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_PATH", pack=False, align=None), offset=0), "pPostalAddress": SimTypePointer(SimStruct({"PostalAddress": SimTypeFixedSizeArray(SimTypePointer(SimTypeChar(label="Char"), offset=0), 6)}, name="ADS_POSTALADDRESS", pack=False, align=None), offset=0), "Timestamp": SimStruct({"WholeSeconds": SimTypeInt(signed=False, label="UInt32"), "EventID": SimTypeInt(signed=False, label="UInt32")}, name="ADS_TIMESTAMP", pack=False, align=None), "BackLink": SimStruct({"RemoteID": SimTypeInt(signed=False, label="UInt32"), "ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_BACKLINK", pack=False, align=None), "pTypedName": SimTypePointer(SimStruct({"ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Level": SimTypeInt(signed=False, label="UInt32"), "Interval": SimTypeInt(signed=False, label="UInt32")}, name="ADS_TYPEDNAME", pack=False, align=None), offset=0), "Hold": SimStruct({"ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Amount": SimTypeInt(signed=False, label="UInt32")}, name="ADS_HOLD", pack=False, align=None), "pNetAddress": SimTypePointer(SimStruct({"AddressType": SimTypeInt(signed=False, label="UInt32"), "AddressLength": SimTypeInt(signed=False, label="UInt32"), "Address": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NETADDRESS", pack=False, align=None), offset=0), "pReplicaPointer": SimTypePointer(SimStruct({"ServerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ReplicaType": SimTypeInt(signed=False, label="UInt32"), "ReplicaNumber": SimTypeInt(signed=False, label="UInt32"), "Count": SimTypeInt(signed=False, label="UInt32"), "ReplicaAddressHints": SimTypePointer(SimStruct({"AddressType": SimTypeInt(signed=False, label="UInt32"), "AddressLength": SimTypeInt(signed=False, label="UInt32"), "Address": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NETADDRESS", pack=False, align=None), offset=0)}, name="ADS_REPLICAPOINTER", pack=False, align=None), offset=0), "pFaxNumber": SimTypePointer(SimStruct({"TelephoneNumber": SimTypePointer(SimTypeChar(label="Char"), offset=0), "NumberOfBits": SimTypeInt(signed=False, label="UInt32"), "Parameters": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_FAXNUMBER", pack=False, align=None), offset=0), "Email": SimStruct({"Address": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Type": SimTypeInt(signed=False, label="UInt32")}, name="ADS_EMAIL", pack=False, align=None), "SecurityDescriptor": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NT_SECURITY_DESCRIPTOR", pack=False, align=None), "pDNWithBinary": SimTypePointer(SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpBinaryValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pszDNString": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_DN_WITH_BINARY", pack=False, align=None), offset=0), "pDNWithString": SimTypePointer(SimStruct({"pszStringValue": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszDNString": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_DN_WITH_STRING", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="ADSVALUE", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"vt": SimTypeShort(signed=False, label="UInt16"), "wReserved1": SimTypeShort(signed=False, label="UInt16"), "wReserved2": SimTypeShort(signed=False, label="UInt16"), "wReserved3": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimUnion({"llVal": SimTypeLongLong(signed=True, label="Int64"), "lVal": SimTypeInt(signed=True, label="Int32"), "bVal": SimTypeChar(label="Byte"), "iVal": SimTypeShort(signed=True, label="Int16"), "fltVal": SimTypeFloat(size=32), "dblVal": SimTypeFloat(size=64), "boolVal": SimTypeShort(signed=True, label="Int16"), "__OBSOLETE__VARIANT_BOOL": SimTypeShort(signed=True, label="Int16"), "scode": SimTypeInt(signed=True, label="Int32"), "cyVal": SimTypeBottom(label="CY"), "date": SimTypeFloat(size=64), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "punkVal": SimTypeBottom(label="IUnknown"), "pdispVal": SimTypeBottom(label="IDispatch"), "parray": SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), "pbVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "piVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "plVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pllVal": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pfltVal": SimTypePointer(SimTypeFloat(size=32), offset=0), "pdblVal": SimTypePointer(SimTypeFloat(size=64), offset=0), "pboolVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "__OBSOLETE__VARIANT_PBOOL": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "pscode": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pcyVal": SimTypePointer(SimTypeBottom(label="CY"), offset=0), "pdate": SimTypePointer(SimTypeFloat(size=64), offset=0), "pbstrVal": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "ppunkVal": SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0), "ppdispVal": SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0), "pparray": SimTypePointer(SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), offset=0), "pvarVal": SimTypePointer(SimTypeBottom(label="VARIANT"), offset=0), "byref": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cVal": SimTypeBottom(label="CHAR"), "uiVal": SimTypeShort(signed=False, label="UInt16"), "ulVal": SimTypeInt(signed=False, label="UInt32"), "ullVal": SimTypeLongLong(signed=False, label="UInt64"), "intVal": SimTypeInt(signed=True, label="Int32"), "uintVal": SimTypeInt(signed=False, label="UInt32"), "pdecVal": SimTypePointer(SimTypeBottom(label="DECIMAL"), offset=0), "pcVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "puiVal": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "pulVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pullVal": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "pintVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "puintVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "Anonymous": SimStruct({"pvRecord": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "pRecInfo": SimTypeBottom(label="IRecordInfo")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="_Anonymous_e__Struct", pack=False, align=None), "decVal": SimTypeBottom(label="DECIMAL")}, name="<anon>", label="None")}, name="VARIANT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pAdsValues", "dwNumValues", "pVariant"]),
        #
        'AdsFreeAdsValues': SimTypeFunction([SimTypePointer(SimStruct({"dwType": SimTypeInt(signed=False, label="ADSTYPEENUM"), "Anonymous": SimUnion({"DNString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "CaseExactString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "CaseIgnoreString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "PrintableString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "NumericString": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "Boolean": SimTypeInt(signed=False, label="UInt32"), "Integer": SimTypeInt(signed=False, label="UInt32"), "OctetString": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_OCTET_STRING", pack=False, align=None), "UTCTime": SimStruct({"wYear": SimTypeShort(signed=False, label="UInt16"), "wMonth": SimTypeShort(signed=False, label="UInt16"), "wDayOfWeek": SimTypeShort(signed=False, label="UInt16"), "wDay": SimTypeShort(signed=False, label="UInt16"), "wHour": SimTypeShort(signed=False, label="UInt16"), "wMinute": SimTypeShort(signed=False, label="UInt16"), "wSecond": SimTypeShort(signed=False, label="UInt16"), "wMilliseconds": SimTypeShort(signed=False, label="UInt16")}, name="SYSTEMTIME", pack=False, align=None), "LargeInteger": SimTypeBottom(label="LARGE_INTEGER"), "ClassName": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "ProviderSpecific": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_PROV_SPECIFIC", pack=False, align=None), "pCaseIgnoreList": SimTypePointer(SimStruct({"Next": SimTypePointer(SimTypeBottom(label="ADS_CASEIGNORE_LIST"), offset=0), "String": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_CASEIGNORE_LIST", pack=False, align=None), offset=0), "pOctetList": SimTypePointer(SimStruct({"Next": SimTypePointer(SimTypeBottom(label="ADS_OCTET_LIST"), offset=0), "Length": SimTypeInt(signed=False, label="UInt32"), "Data": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_OCTET_LIST", pack=False, align=None), offset=0), "pPath": SimTypePointer(SimStruct({"Type": SimTypeInt(signed=False, label="UInt32"), "VolumeName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Path": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_PATH", pack=False, align=None), offset=0), "pPostalAddress": SimTypePointer(SimStruct({"PostalAddress": SimTypeFixedSizeArray(SimTypePointer(SimTypeChar(label="Char"), offset=0), 6)}, name="ADS_POSTALADDRESS", pack=False, align=None), offset=0), "Timestamp": SimStruct({"WholeSeconds": SimTypeInt(signed=False, label="UInt32"), "EventID": SimTypeInt(signed=False, label="UInt32")}, name="ADS_TIMESTAMP", pack=False, align=None), "BackLink": SimStruct({"RemoteID": SimTypeInt(signed=False, label="UInt32"), "ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_BACKLINK", pack=False, align=None), "pTypedName": SimTypePointer(SimStruct({"ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Level": SimTypeInt(signed=False, label="UInt32"), "Interval": SimTypeInt(signed=False, label="UInt32")}, name="ADS_TYPEDNAME", pack=False, align=None), offset=0), "Hold": SimStruct({"ObjectName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Amount": SimTypeInt(signed=False, label="UInt32")}, name="ADS_HOLD", pack=False, align=None), "pNetAddress": SimTypePointer(SimStruct({"AddressType": SimTypeInt(signed=False, label="UInt32"), "AddressLength": SimTypeInt(signed=False, label="UInt32"), "Address": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NETADDRESS", pack=False, align=None), offset=0), "pReplicaPointer": SimTypePointer(SimStruct({"ServerName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "ReplicaType": SimTypeInt(signed=False, label="UInt32"), "ReplicaNumber": SimTypeInt(signed=False, label="UInt32"), "Count": SimTypeInt(signed=False, label="UInt32"), "ReplicaAddressHints": SimTypePointer(SimStruct({"AddressType": SimTypeInt(signed=False, label="UInt32"), "AddressLength": SimTypeInt(signed=False, label="UInt32"), "Address": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NETADDRESS", pack=False, align=None), offset=0)}, name="ADS_REPLICAPOINTER", pack=False, align=None), offset=0), "pFaxNumber": SimTypePointer(SimStruct({"TelephoneNumber": SimTypePointer(SimTypeChar(label="Char"), offset=0), "NumberOfBits": SimTypeInt(signed=False, label="UInt32"), "Parameters": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_FAXNUMBER", pack=False, align=None), offset=0), "Email": SimStruct({"Address": SimTypePointer(SimTypeChar(label="Char"), offset=0), "Type": SimTypeInt(signed=False, label="UInt32")}, name="ADS_EMAIL", pack=False, align=None), "SecurityDescriptor": SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="ADS_NT_SECURITY_DESCRIPTOR", pack=False, align=None), "pDNWithBinary": SimTypePointer(SimStruct({"dwLength": SimTypeInt(signed=False, label="UInt32"), "lpBinaryValue": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pszDNString": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_DN_WITH_BINARY", pack=False, align=None), offset=0), "pDNWithString": SimTypePointer(SimStruct({"pszStringValue": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pszDNString": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="ADS_DN_WITH_STRING", pack=False, align=None), offset=0)}, name="<anon>", label="None")}, name="ADSVALUE", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["pAdsValues", "dwNumValues"]),
        #
        'BinarySDToSecurityDescriptor': SimTypeFunction([SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"vt": SimTypeShort(signed=False, label="UInt16"), "wReserved1": SimTypeShort(signed=False, label="UInt16"), "wReserved2": SimTypeShort(signed=False, label="UInt16"), "wReserved3": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimUnion({"llVal": SimTypeLongLong(signed=True, label="Int64"), "lVal": SimTypeInt(signed=True, label="Int32"), "bVal": SimTypeChar(label="Byte"), "iVal": SimTypeShort(signed=True, label="Int16"), "fltVal": SimTypeFloat(size=32), "dblVal": SimTypeFloat(size=64), "boolVal": SimTypeShort(signed=True, label="Int16"), "__OBSOLETE__VARIANT_BOOL": SimTypeShort(signed=True, label="Int16"), "scode": SimTypeInt(signed=True, label="Int32"), "cyVal": SimTypeBottom(label="CY"), "date": SimTypeFloat(size=64), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "punkVal": SimTypeBottom(label="IUnknown"), "pdispVal": SimTypeBottom(label="IDispatch"), "parray": SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), "pbVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "piVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "plVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pllVal": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pfltVal": SimTypePointer(SimTypeFloat(size=32), offset=0), "pdblVal": SimTypePointer(SimTypeFloat(size=64), offset=0), "pboolVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "__OBSOLETE__VARIANT_PBOOL": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "pscode": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pcyVal": SimTypePointer(SimTypeBottom(label="CY"), offset=0), "pdate": SimTypePointer(SimTypeFloat(size=64), offset=0), "pbstrVal": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "ppunkVal": SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0), "ppdispVal": SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0), "pparray": SimTypePointer(SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), offset=0), "pvarVal": SimTypePointer(SimTypeBottom(label="VARIANT"), offset=0), "byref": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cVal": SimTypeBottom(label="CHAR"), "uiVal": SimTypeShort(signed=False, label="UInt16"), "ulVal": SimTypeInt(signed=False, label="UInt32"), "ullVal": SimTypeLongLong(signed=False, label="UInt64"), "intVal": SimTypeInt(signed=True, label="Int32"), "uintVal": SimTypeInt(signed=False, label="UInt32"), "pdecVal": SimTypePointer(SimTypeBottom(label="DECIMAL"), offset=0), "pcVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "puiVal": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "pulVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pullVal": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "pintVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "puintVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "Anonymous": SimStruct({"pvRecord": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "pRecInfo": SimTypeBottom(label="IRecordInfo")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="_Anonymous_e__Struct", pack=False, align=None), "decVal": SimTypeBottom(label="DECIMAL")}, name="<anon>", label="None")}, name="VARIANT", pack=False, align=None), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pSecurityDescriptor", "pVarsec", "pszServerName", "userName", "passWord", "dwFlags"]),
        #
        'SecurityDescriptorToBinarySD': SimTypeFunction([SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"vt": SimTypeShort(signed=False, label="UInt16"), "wReserved1": SimTypeShort(signed=False, label="UInt16"), "wReserved2": SimTypeShort(signed=False, label="UInt16"), "wReserved3": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimUnion({"llVal": SimTypeLongLong(signed=True, label="Int64"), "lVal": SimTypeInt(signed=True, label="Int32"), "bVal": SimTypeChar(label="Byte"), "iVal": SimTypeShort(signed=True, label="Int16"), "fltVal": SimTypeFloat(size=32), "dblVal": SimTypeFloat(size=64), "boolVal": SimTypeShort(signed=True, label="Int16"), "__OBSOLETE__VARIANT_BOOL": SimTypeShort(signed=True, label="Int16"), "scode": SimTypeInt(signed=True, label="Int32"), "cyVal": SimTypeBottom(label="CY"), "date": SimTypeFloat(size=64), "bstrVal": SimTypePointer(SimTypeChar(label="Char"), offset=0), "punkVal": SimTypeBottom(label="IUnknown"), "pdispVal": SimTypeBottom(label="IDispatch"), "parray": SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), "pbVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "piVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "plVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pllVal": SimTypePointer(SimTypeLongLong(signed=True, label="Int64"), offset=0), "pfltVal": SimTypePointer(SimTypeFloat(size=32), offset=0), "pdblVal": SimTypePointer(SimTypeFloat(size=64), offset=0), "pboolVal": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "__OBSOLETE__VARIANT_PBOOL": SimTypePointer(SimTypeShort(signed=True, label="Int16"), offset=0), "pscode": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "pcyVal": SimTypePointer(SimTypeBottom(label="CY"), offset=0), "pdate": SimTypePointer(SimTypeFloat(size=64), offset=0), "pbstrVal": SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), "ppunkVal": SimTypePointer(SimTypeBottom(label="IUnknown"), offset=0), "ppdispVal": SimTypePointer(SimTypeBottom(label="IDispatch"), offset=0), "pparray": SimTypePointer(SimTypePointer(SimStruct({"cDims": SimTypeShort(signed=False, label="UInt16"), "fFeatures": SimTypeShort(signed=False, label="UInt16"), "cbElements": SimTypeInt(signed=False, label="UInt32"), "cLocks": SimTypeInt(signed=False, label="UInt32"), "pvData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "rgsabound": SimTypePointer(SimStruct({"cElements": SimTypeInt(signed=False, label="UInt32"), "lLbound": SimTypeInt(signed=True, label="Int32")}, name="SAFEARRAYBOUND", pack=False, align=None), offset=0)}, name="SAFEARRAY", pack=False, align=None), offset=0), offset=0), "pvarVal": SimTypePointer(SimTypeBottom(label="VARIANT"), offset=0), "byref": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cVal": SimTypeBottom(label="CHAR"), "uiVal": SimTypeShort(signed=False, label="UInt16"), "ulVal": SimTypeInt(signed=False, label="UInt32"), "ullVal": SimTypeLongLong(signed=False, label="UInt64"), "intVal": SimTypeInt(signed=True, label="Int32"), "uintVal": SimTypeInt(signed=False, label="UInt32"), "pdecVal": SimTypePointer(SimTypeBottom(label="DECIMAL"), offset=0), "pcVal": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "puiVal": SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), "pulVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "pullVal": SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0), "pintVal": SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), "puintVal": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), "Anonymous": SimStruct({"pvRecord": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "pRecInfo": SimTypeBottom(label="IRecordInfo")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="_Anonymous_e__Struct", pack=False, align=None), "decVal": SimTypeBottom(label="DECIMAL")}, name="<anon>", label="None")}, name="VARIANT", pack=False, align=None), SimTypePointer(SimTypePointer(SimStruct({"Revision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "Control": SimTypeShort(signed=False, label="UInt16"), "Owner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Group": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Sacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0), "Dacl": SimTypePointer(SimStruct({"AclRevision": SimTypeChar(label="Byte"), "Sbz1": SimTypeChar(label="Byte"), "AclSize": SimTypeShort(signed=False, label="UInt16"), "AceCount": SimTypeShort(signed=False, label="UInt16"), "Sbz2": SimTypeShort(signed=False, label="UInt16")}, name="ACL", pack=False, align=None), offset=0)}, name="SECURITY_DESCRIPTOR", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["vVarSecDes", "ppSecurityDescriptor", "pdwSDLength", "pszServerName", "userName", "passWord", "dwFlags"]),
    }

lib.set_prototypes(prototypes)



import angr


class gethostbyname(angr.SimProcedure):
    def run(self, name):
        malloc = angr.SIM_PROCEDURES["libc"]["malloc"]
        place = self.inline_call(malloc, 32).ret_expr
        self.state.memory.store(
            place, self.state.solver.BVS("h_name", 64, key=("api", "gethostbyname", "h_name")), endness="Iend_LE"
        )
        self.state.memory.store(
            place, self.state.solver.BVS("h_aliases", 64, key=("api", "gethostbyname", "h_aliases")), endness="Iend_LE"
        )
        self.state.memory.store(
            place,
            self.state.solver.BVS("h_addrtype", 64, key=("api", "gethostbyname", "h_addrtype")),
            endness="Iend_LE",
        )
        self.state.memory.store(
            place, self.state.solver.BVS("h_length", 64, key=("api", "gethostbyname", "h_length")), endness="Iend_LE"
        )
        self.state.memory.store(
            place,
            self.state.solver.BVS("h_addr_list", 64, key=("api", "gethostbyname", "h_addr_list")),
            endness="Iend_LE",
        )
        return place



import angr


class ReturnUnconstrained(angr.SimProcedure):
    ARGS_MISMATCH = True

    def run(self, *args, **kwargs):  # pylint:disable=arguments-differ
        # pylint:disable=attribute-defined-outside-init

        return_val = kwargs.pop("return_val", None)
        if return_val is None:
            # code duplicated to syscall_stub
            size = self.prototype.returnty.size
            # ummmmm do we really want to rely on this behavior?
            if size is NotImplemented:
                o = None
            else:
                o = self.state.solver.Unconstrained(
                    "unconstrained_ret_%s" % self.display_name, size, key=("api", "?", self.display_name)
                )
        else:
            o = return_val

        return o



#!/usr/bin/env python3
__package__ = __package__ or "tests.analyses.cfg"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ...common import bin_location

test_location = os.path.join(bin_location, "tests")
arches = {"i386", "x86_64"}


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestCfgGetAnyNode(unittest.TestCase):
    def test_cfg_get_any_node(self):
        for arch in arches:
            self.run_cfg_get_any_node(arch)

    def run_cfg_get_any_node(self, arch):
        test_file = os.path.join(test_location, arch, "hello_world")
        proj = angr.Project(test_file, auto_load_libs=False)
        cfg = proj.analyses.CFGFast()

        for node1 in cfg.nodes():
            if node1.size == 0:
                node2 = cfg.get_any_node(addr=node1.addr, anyaddr=True)
                assert node2 is not None


if __name__ == "__main__":
    unittest.main()
